<?php
/**
 * Project:
 * Contenido Content Management System
 *
 * Description:
 * Contenido Template Engine
 *
 * Requirements:
 * @con_php_req 5.0
 *
 *
 * @package    Contenido Backend classes
 * @version    1.2.2
 * @author     Jan Lengowski
 * @copyright  four for business AG <www.4fb.de>
 * @license    http://www.contenido.org/license/LIZENZ.txt
 * @link       http://www.4fb.de
 * @link       http://www.contenido.org
 * @since      file available since contenido release <= 4.6
 *
 * {@internal
 *   created unknown
 *   modified 2008-06-30, Frederic Schneider, add security fix
 *   modified 2008-09-03  Timo Trautmann, DB Object is only created if necessary
 *
 *   $Id: class.template.php 790 2008-09-04 16:34:28Z timo.trautmann $:
 * }}
 *
 */


defined('CON_FRAMEWORK') or die('Illegal call');


/**
 * Template class, based on class.template.php from Contenido
 *
 * Light template mechanism
 *
 * @author     Jan Lengowski <Jan.Lengowski@4fb.de>
 * @copyright  four for business <http://www.4fb.de>
 * @author     Stefan Jelner (Optimizations)
 * @author     Murat Purc <murat@purc.de> Adaption to PHP5 and some modifications
 * @version    1.2
 */
class ConMp_Template
{

    /**
     * Configuration array
     * @var array
     */
    private $_cfg;

    /**
     * Template note
     * @var  string
     */
    private $_note = "<!-- Generated by Contenido %s -->";

    /**
     * Static needles
     * @var  array
     */
    private $_needles = array();

    /**
     * Static replacements
     * @var  array
     */
    private $_replacements = array();

    /**
     * Dynamic needles
     * @var  array
     */
    private $_dynNeedles = array();

    /**
     * Dynamic replacements
     * @var  array
     */
    private $_dynReplacements = array();

    /**
     * Dynamic counter
     * @var int
     */
    private $_dynCnt = 0;

    /**
     * Tags array (for dynamic blocks);
     * @var array
     */
    private $_tags = array('static' => '{%s}', 'start' => '<!-- BEGIN:BLOCK -->', 'end' => '<!-- END:BLOCK -->');


    /**
     * Constructor function
     * @return void
     */
    public function __constructor(array $tags = array())
    {
        $this->_cfg = $GLOBALS['cfg'];

        if (count($tags) > 0) {
            $this->_tags = $tags;
        }

        $this->setEncoding('');
        $this->setDomain('contenido');
    }


    /**
     * Sets the gettext domain to use for translations in a template
     *
     * @param  string  $sDomain  Sets the domain to use for template translations
     * @return none
     */
    public function setDomain($sDomain)
    {
        $this->_sDomain = $sDomain;
    }


    /**
     * Set Templates placeholders and values
     *
     * With this method you can replace the placeholders
     * in the static templates with dynamic data.
     *
     * @param   string  $which 's' for Static or else dynamic
     * @param   string  $needle Placeholder
     * @param   string  $replacement Replacement String
     * @return  void
     */
    public function set($which = 's', $needle, $replacement)
    {
        if ($which == 's') {
            // static
            $this->_needles[] = sprintf($this->_tags['static'], $needle);
            $this->_replacements[] = $replacement;
        } else {
            // dynamic
            $this->_dynNeedles[$this->_dynCnt][] = sprintf($this->_tags['static'], $needle);
            $this->_dynReplacements[$this->_dynCnt][] = $replacement;
        }
    }


    /**
     * Sets an encoding for the template's head block.
     *
     * @param   string  $encoding  Encoding to set
     * @return  void
     */
    public function setEncoding($encoding)
    {
        $this->_encoding = $encoding;
    }


    /**
     * Iterate internal counter by one
     *
     * @return void
     */
    public function next()
    {
        $this->_dynCnt++;
    }


    /**
     * Reset template data
     *
     * @return void
     */
    public function reset()
    {
        $this->_dynCnt          = 0;
        $this->_needles         = array();
        $this->_replacements    = array();
        $this->_dynNeedles      = array();
        $this->_dynReplacements = array();
    }


    /**
     * Generate the template and
     * print/return it. (do translations sequentially to save memory!!!)
     *
     * @param   string  $template  Either template string or file
     * @param   bool    $return    Return or print template
     * @param   bool    $note      Echo "Generated by ... " Comment
     * @return  string  Complete template string
     */
    public function generate($template, $return = false, $note = false)
    {
        $return = (bool) $return;
        $note   = (bool) $note;

        //check if the template is a file or a string
        if (!@is_file($template)) {
            //template is a string (it is a reference to save memory!!!)
            $content = & $template;
        } else {
            //template is a file
            $content = file_get_contents($template);
        }

        $content = (($note) ? sprintf($this->_note, $this->_cfg['version']) : '') . $content;

        $pieces = array();

        //replace i18n strings before replacing other placeholders
        $this->_replacei18n($content, 'i18n');
        $this->_replacei18n($content, 'trans');

        // quote start and end of dynamic blocks
        $quotedStart = preg_quote($this->_tags['start'], '/');
        $quotedEnd   = preg_quote($this->_tags['end'], '/');

        //if content has dynamic blocks
        if (preg_match('/^.*' . $quotedStart . '.*?' . $quotedEnd . '.*$/s', $content)) {
            // split everything into an array
            preg_match_all('/^(.*)' . $quotedStart . '(.*?)' . $quotedEnd . '(.*)$/s', $content, $pieces);
            // safe memory
            array_shift($pieces);
            $content = '';
            // now combine pieces together

            //start block
            $content .= str_replace($this->_needles, $this->_replacements, $pieces[0][0]);
            unset($pieces[0][0]);

            //generate dynamic blocks
            for ($a = 0; $a < $this->_dynCnt; $a ++) {
                $content .= str_replace($this->_dynNeedles[$a], $this->_dynReplacements[$a], $pieces[1][0]);
            }
            unset($pieces[1][0]);

            //end block
            $content .= str_replace($this->_needles, $this->_replacements, $pieces[2][0]);
            unset($pieces[2][0]);
        } else {
            $content = str_replace($this->_needles, $this->_replacements, $content);
        }

        if ($this->_encoding != '') {
            $content = str_replace('</head>', '<meta http-equiv="Content-Type" content="text/html; charset='.$this->_encoding.'">'."\n".'</head>', $content);
        }

        if ($return) {
            return $content;
        } else {
            echo $content;
        }
    }


   /**
    * Replaces a named function with the translated variant
    *
    * @param  string  $template      Contents of the template to translate (it is reference to save memory!!!)
    * @param  string  $functionName  Name of the translation function (e.g. i18n)
    */
    private function _replacei18n(& $template, $functionName) {

        // Be sure that php code stays unchanged
        $php_matches = array();
        if (preg_match_all('/<\?(php)?((.)|(\s))*?\?>/i', $template, $php_matches)) {
            $x = 0;
            foreach ($php_matches[0] as $php_match) {
                $x++;
                $template = str_replace($php_match , '{PHP#' . $x . '#PHP}', $template);
                $container[$x] = $php_match;
            }
        }

        // If template contains functionName + parameter store all matches
        $quotedFunctionName = preg_quote($functionName, '/');
        $matches = array();
        preg_match_all('/' . $quotedFunctionName . "\\(([\\\"\\'])(.*?)\\1\\)/s", $template, $matches);

        $matches = array_values(array_unique($matches[2]));
        for ($a = 0; $a < count($matches); $a ++) {
            $template = preg_replace('/' . $quotedFunctionName . "\\([\\\"\\']" . preg_quote($matches[$a], '/') . "[\\\"\\']\\)/s", i18n($matches[$a]), $template);
        }

        // Change back php placeholder
        if (is_array($container)) {
            foreach ($container as $x => $php_match) {
                $template = str_replace('{PHP#' . $x . '#PHP}' , $php_match, $template);
            }
        }
    }


    /**
     * Returns the name for the given template
     *
     * @deprecated  This function is in the wrong place
     */
    public function getTemplateName($tplid)
    {
        throw new Exception('Call of deprecated ConMp_Template->getTemplateName()');
    }


    /**
     * Returns the desription for the given template
     *
     * @deprecated  This function is in the wrong place
     */
    public function getTemplateDescription($tplid)
    {
        throw new Exception('Call of deprecated ConMp_Template->getTemplateDescription()');
    }


    /**
     * Returns the name for the given template using the template configuration ID
     *
     * @deprecated  This function is in the wrong place
     */
    public function getTemplateNameFromTPLCFG($tplcfg)
    {
        throw new Exception('Call of deprecated ConMp_Template->getTemplateNameFromTPLCFG()');
    }

}
